{ config, lib, ... }:
# { config, lib, inputs, ... }:

{

  packer = {
    required_plugins = {
      hcloud = {
        source  = "github.com/hetznercloud/hcloud";
        version = "~> 1";
      };
    };
  };
  
  variable = {

    hcloud_api_token = {
      type = "string";
      description = "[Hetzner Cloud API Token](https://docs.hetzner.com/cloud/api/getting-started/generating-api-token)";
      sensitive = true;
    };

    nixos_version = {
      type    = "string";
      default = "23.11";
    };
    
    arch = {
      type    = "string";
      default = "aarch64";  # x86_64
    };

    # https://docs.hetzner.com/cloud/servers/overview/#pricing
    server_type = {
      type    = "string";
      # can use images on servers bigger than the build server
      # but not smaller, so this is the smallest option,
      # in this case for arm architecture
      default = "cax11";
    };

    # https://docs.hetzner.com/cloud/general/locations/
    server_location = {
      type    = "string";
      default = "nbg1";
    };

  };

  locals = {
    image = "https://github.com/nixos/nixos/releases/download/\${var.nixos_version}/hcloud-\${var.arch}.raw.xz";
    # image = "https://channels.nixos.org/nixos-\${var.nixos_version}/latest-nixos-minimal-\${var.arch}-linux.iso";
  };

  # https://developer.hashicorp.com/packer/integrations/hetznercloud/hcloud/latest/components/builder/hcloud
  source."hcloud"."nixos" = {
    token        = "\${var.hcloud_api_token}";
    rescue       = "linux64";
    image        = "ubuntu-22.04";
    location     = "\${var.server_location}";
    server_type  = "\${var.server_type}";
    ssh_keys     = [];
    # user_data_file = "";
    # TODO: use flake input instead
    user_data    = ''
      #cloud-config
      runcmd:
      - curl https://raw.githubusercontent.com/elitak/nixos-infect/master/nixos-infect | PROVIDER=hetznercloud NIX_CHANNEL=nixos-''${var.nixos_version} bash 2>&1 | tee /tmp/infect.log
    ssh_username = "root";
  
    snapshot_name   = "nixos system disk - \${var.arch} - \${var.nixos_version}";
    snapshot_labels = {
      type    = "infra";
      os      = "nixos";
      version = "\${var.nixos_version}";
      arch    = "\${var.arch}";
    };
  };
  
  build = {
    # can e.g. specify multiple for different architectures setting respective
    # default server types, to automatically build for multiple architectures
    sources = ["source.hcloud.nixos"];
  
    provisioner."shell" = {
      env = {
        BUILDER = "packer";
      };

      # scripts = [
      #   "os-setup.sh"
      # ];
      # or:
      # inline = [
      #   # "apt-get install -y wget"
      #   # "wget -O /tmp/nixos.raw.xz \${local.image}"
      #   # "xz -d -c /tmp/nixos.raw.xz | dd of=/dev/sda && sync"
      #   "curl https://raw.githubusercontent.com/elitak/nixos-infect/master/nixos-infect | NIX_CHANNEL=nixos-\${var.nixos_version} bash -x"
      #   # "export NIX_CHANNEL=nixos-\${var.nixos_version}; ${lib.readFile "${inputs.nixos-infect}/nixos-infect"}"
      #   # additional inspiration: https://github.com/terranix/terranix-hcloud/blob/main/module/nixserver.nix#L95-L118
      # ];
    };
  };

}

# Additionally you could create a file containing

# arch            = "aarch64"
# server_type     = "cax11"
# server_location = "nbg1"


# # First you need set API Token
# ```sh
# export HCLOUD_TOKEN=${TOKEN}

# # Upload image
# packer init .
# packer build .
# # Save the image ID
# export IMAGE_ID=<image-id-in-packer-output>
# ```

# packer build -var arch=aarch64 -var server_type=cax11 -var server_location=nbg1 .


# further settings of note:
# - https://github.com/input-output-hk/nomad-driver-nix/blob/main/flake.nix#L106-L108
# - https://xeiaso.net/blog/paranoid-nixos-2021-07-18/
